from adls_management.adls_management import connection
from adls_management.utils import settings, messages


class ADLSFolderManagement:

    def __init__(self, configuration_file):
        # use provided connection config file
        self.settings = settings.GenericSettings(configuration_file=configuration_file)
        self.settings.get_config()

        self.service_client, self.file_system_client, self.blob_service_client, self.container_client, self.sas_token \
            = connection.ConnectionManagement(self.settings).create_connection(
            storage_account_name=self.settings.storage_account_name
            , storage_account_key=self.settings.storage_account_key
            , container=self.settings.storage_container
        )

    def create_directory(self, directory):
        try:
            self.file_system_client.create_directory(directory=directory)
        except Exception as e:
            print(e)
            return messages.message["directory_creation_error"]
        return messages.message["ok"]

    def delete_directory(self, directory):
        try:
            self.file_system_client.delete_directory(directory=directory)
        except Exception as e:
            print(e)
            return messages.message["directory_removal_error"]
        return messages.message["ok"]



import re

import setuptools

# https://packaging.python.org/tutorials/packaging-projects/

with open("README.md", "r") as fh:
    long_description = fh.read()

version_file = 'resources/version.py'
tmp_version_file = 'temp/_tmp_version.tmp'

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open(version_file, 'rb') as f:
    for line in f:
        line = line.strip()
        if line:
            line = line.decode('utf-8')
            reg_comment = re.compile(r'^#.*')
            m = reg_comment.search(line)
            if not m:
                result_search = _version_re.search(line)
                version = result_search.group(1)
                version = version.strip('\"')
                main_version, sub_version, fix_version = version.split(".")
                fix_number = int(fix_version) + 1
                new_version = main_version + "." + sub_version + "." + str(fix_number)

                with open(tmp_version_file, 'wb') as t:
                    out_line = '__version__ = "' + new_version + '"\n'
                    t.write(out_line.encode('utf-8'))

setuptools.setup(
    name='adls-management',
    version=new_version,
    author='Jac. Beekers',
    author_email='beekersjac@gmail.com',
    description="metadata lake file management",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/jacbeekers/metalake-file-management',
    packages=setuptools.find_packages(),
    install_requires=[
        'jsonschema',
        'azure-storage-file-datalake'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.6'
)



import os, uuid, sys
from azure.storage.filedatalake import DataLakeServiceClient
from azure.core._match_conditions import MatchConditions
from azure.storage.filedatalake._models import ContentSettings
from adls_management.utils import settings
from azure.storage.blob import BlobClient, BlobServiceClient, generate_account_sas
from azure.storage.blob import ResourceTypes, AccountSasPermissions
from datetime import datetime, timedelta


class ConnectionManagement:

    def __init__(self, settings):
        self.service_client = None
        self.file_system_client = None
        self.settings = settings
        self.blob_service_client = None
        self.container_client = None
        self.sas_token = None

    def create_connection(self, storage_account_name, storage_account_key, container):
        try:
            dfs_url="{}://{}.dfs.core.windows.net".format("https", storage_account_name)
            blob_url="{}://{}.blob.core.windows.net/".format("https", storage_account_name)
            # print("ADLS URL:", dfs_url)
            # print("Blob URL:", blob_url)
            self.service_client = DataLakeServiceClient(account_url=dfs_url, credential=storage_account_key)
            # print("Getting file_system_client...")
            self.file_system_client = self.service_client.get_file_system_client(
                file_system=self.settings.storage_container
            )
            # print("Getting blob_service_client...")
            connect_string="DefaultEndpointsProtocol=https;AccountName=" + storage_account_name + ";AccountKey="\
                           + storage_account_key + ";EndpointSuffix=core.windows.net"
            self.blob_service_client = BlobServiceClient.from_connection_string(
                    conn_str=connect_string)
            # Create sas token for blob

            self.sas_token = generate_account_sas(
                account_name=self.blob_service_client.account_name,
                account_key=storage_account_key,
                resource_types = ResourceTypes(service=True, object=True, container=True),
                permission = AccountSasPermissions(read=True, write=True, delete=True, list=True, add=True, create=True),
                start = datetime.now() - timedelta(hours=1),
                expiry = datetime.utcnow() + timedelta(hours=4)  # Token valid for 4 hours
            )
            self.container_client = self.blob_service_client.get_container_client(container)
            # print("returning references.")
            return self.service_client\
                , self.file_system_client\
                , self.blob_service_client\
                , self.container_client\
                , self.sas_token

        except Exception as e:
            print(e)
            return None, None, None, None, None



import json

from adls_management.utils import messages


class GenericSettings:
    """
    Some generic utilities, e.g. reading the config.json
    """
    code_version = "0.1.0"

    def __init__(self, configuration_file="resources/connection_config.json"):
        # config.json settings
        self.main_config_file = configuration_file
        self.meta_version = None
        self.output_directory = None
        self.log_config = None
        self.suppress_azure_call = False
        self.azure_http_proxy = None
        self.azure_https_proxy = None
        self.storage_account_name = None
        self.storage_account_key = None
        self.storage_container = None
        self.azure_secrets = None
        self.file_locations = None
        self.incoming = None
        self.todo = None
        self.busy = None
        self.done = None
        self.redo = None
        self.hist = None
        self.process_locations = False
        self.download_location = "jsons"

    def get_config(self):
        """
            get the main configuration settings. default file is resources/config.json
        """
        module = __name__ + ".get_config"
        result = messages.message["undetermined"]

        try:
            with open(self.main_config_file) as config:
                data = json.load(config)
                # self.schema_directory = self.base_schema_folder + self.meta_version + "/"
                if "azure_secrets" in data:
                    self.azure_secrets = data["azure_secrets"]
                if "file_locations" in data:
                    self.file_locations = data["file_locations"]
                if "suppress_azure_call" in data:
                    if data["suppress_azure_call"] == "True":
                        self.suppress_azure_call = True
                    elif data["suppress_azure_call"] == "False":
                        self.suppress_azure_call = False
                    else:
                        print("Incorrect config value >" + data["suppress_azure_call"]
                              + "< for suppress_azure_call. Must be True or False. Will default to False")
                        self.suppress_azure_call = False
                if "download_location" in data:
                    self.download_location = data["download_location"]

            result = messages.message["ok"]

        except FileNotFoundError:
            print("FATAL:", module, "could find not main configuration file >" + self.main_config_file + "<.")
            return messages.message["main_config_not_found"]

        if self.azure_secrets is None:
            print("azure secrets are unknown. Please set azure_secrets in the main config file.")
        else:
            azure_secrets_result = self.get_azure_secrets(self.azure_secrets)
            if azure_secrets_result["code"] != "OK":
                # print("get_azure_secrets returned: " + azure_secrets_result["code"], module)
                return azure_secrets_result

        if self.file_locations is None:
            print("file locations are unknown. Please set file_locations in the main config file.")
        else:
            file_locations_result = self.get_file_locations(self.file_locations)
            if file_locations_result["code"] != "OK":
                # print("get_file_locations returned: " + file_locations_result["code"], module)
                return file_locations_result

        return result

    def get_azure_proxy(self):

        if self.azure_http_proxy == "None":
            self.azure_http_proxy = None
        if self.azure_https_proxy == "None":
            self.azure_https_proxy = None

        proxies = {
            "http": self.azure_http_proxy,
            "https": self.azure_https_proxy
        }
        return proxies

    def get_file_locations(self, config_file):
        if config_file is None:
            return messages.message["not_provided"]
        try:
            with open(config_file) as locations:
                data = json.load(locations)
                self.incoming = data["incoming"]
                self.todo = data["todo"]
                self.busy = data["busy"]
                self.done = data["done"]
                self.redo = data["redo"]
                self.hist = data["hist"]
                self.process_locations = True
        except FileNotFoundError as e:
            print("File with file_locations >" + self.file_locations + "< could not be found.")
            return messages.message["file_locations_not_found"]

        return messages.message["ok"]

    def get_azure_secrets(self, azure_secrets="resources/azure.secrets"):
        module = __name__ + ".get_azure_secrets"

        try:
            with open(azure_secrets) as azure:
                data = json.load(azure)
                result = self.determine_azure_secrets(data)

                if result["code"] != "OK":
                    print("ERROR: Determine azure secrets returned: " + result["code"])
                    return result
        except FileNotFoundError:
            print("ERROR: Cannot find provided azure_secrets file >" + self.azure_secrets + "<."
                  , module)
            return messages.message["azure_secrets_not_found"]

        return messages.message["ok"]

    def determine_azure_secrets(self, data):
        module = __name__ + ".determine_azure_secrets"

        if "meta_version" in data:
            main_meta_version = data["meta_version"][:3]
            if main_meta_version != "0.1":
                print("Unsupported meta_version >" + data["meta_version"] + "<."
                      , module)
                return messages.message["unsupported_meta_version_azure_secrets"]
        else:
            print("Backward compatible azure secrets file detected. Please update to a later version."
                  , module)

        self.storage_account_name = data["storage_account_name"]
        self.storage_account_key = data["storage_account_key"]
        self.storage_container = data["container"]
        # print("container: ", self.storage_container)
        if "azure_http_proxy" in data:
            self.azure_http_proxy = data["azure_http_proxy"]
        #    print("HTTP Proxy for Azure taken from azure secrets file: "
        #          + self.azure_http_proxy, module)
        # else:
        #    print("No HTTP Proxy for Azure found in azure secrets file. "
        #          + "This is OK if no proxy is needed or has been set through the environment variable HTTP_PROXY"
        #          , module)
        if "azure_https_proxy" in data:
            self.azure_https_proxy = data["azure_https_proxy"]
        #    print("HTTPS Proxy for Azure taken from azure secrets file: "
        #          + self.azure_https_proxy, module)
        # else:
        #    print("No HTTPS Proxy for Azure found in azure secrets file. "
        #          + "This is OK if no proxy is needed or has been set through the environment variable HTTPS_PROXY"
        #          , module)

        return messages.message["ok"]
